#!/usr/bin/env python3
###
### Script to check water conservation in the IPSL coupled model
###
##  Warning, to install, configure, run, use any of included software or
##  to read the associated documentation you'll need at least one (1)
##  brain in a reasonably working order. Lack of this implement will
##  void any warranties (either express or implied).  Authors assumes
##  no responsability for errors, omissions, data loss, or any other
##  consequences caused directly or indirectly by the usage of his
##  software by incorrectly or partially configured personal
##
##
## SVN information
#  $Author$
#  $Date$
#  $Revision$
#  $Id$
#  $HeadURL$


###
## Import system modules
import sys, os, shutil, subprocess, platform
import numpy as np
import configparser, re
from pathlib import Path

# Check python version
if sys.version_info < (3, 8, 0) :
    print ( f'Python version : {platform.python_version()}' ) 
    raise Exception ( "Minimum Python version is 3.8" )

## Import local module
import WaterUtils as wu

## Creates parser for reading .ini input file
config = configparser.ConfigParser (interpolation=configparser.ExtendedInterpolation() )
config.optionxform = str # To keep capitals

## Experiment parameters
ATM=None ; ATM_HIS='latlon' ; SRF_HIS='latlon' ; ORCA=None ; NEMO=None ; OCE_relax=False ; OCE_icb=False ; Coupled=False ; Routing=None ;  TarRestartPeriod_beg=None ; TarRestartPeriod_end=None ; Comment=None ; Period=None ; Title=None

##
ARCHIVE=None ; STORAGE=None ; SCRATCHDIR=None ; R_IN=None ; rebuild=None
TmpDir=None ; RunDir=None ; FileOut=None ; dir_ATM_his=None ; dir_SRF_his=None ; dir_OCE_his=None ; dir_ICE_his=None ; FileCommon=None
file_ATM_his=None ; file_SRF_his=None ; file_RUN_his=None ; file_OCE_his=None ;  file_ICE_his=None ; file_OCE_sca=None
tar_restart_beg=None ; tar_restart_end=None ; file_ATM_beg=None ; file_ATM_end=None ; file_DYN_beg=None ; file_DYN_end=None ; file_SRF_beg=None ; file_SRF_end=None
file_RUN_beg=None ; file_RUN_end=None ; file_RUN_end=None ; file_OCE_beg=None ; file_ICE_beg=None ; file_OCE_beg=None ; file_OCE_end=None ; file_OCE_srf=None ; file_OCE_sca=None ; file_ICE_beg=None ; file_OCE_end=None ; file_ICE_end=None

# Read command line arguments
print ( "Name of Python script:", sys.argv[0] )
IniFile = sys.argv[1]
# Text existence of IniFile
if not os.path.exists (IniFile ) :
    raise FileExistsError ( f"File not found : {IniFile = }" )

if 'full' in IniFile : FullIniFile = IniFile
else                 : FullIniFile = 'full_' + IniFile
    
print ("Input file : ", IniFile )
config.read (IniFile)
FullIniFile = 'full_' + IniFile

## Reading config file
for Section in ['Experiment', 'libIGCM', 'Files', 'Physics' ] :
   if Section in config.keys () : 
        print ( f'Reading [{Section}]' )
        for VarName in config[Section].keys() :
            locals()[VarName] = config[Section][VarName]
            exec ( f'{VarName} = wu.setBool ({VarName})' )
            exec ( f'{VarName} = wu.setNum  ({VarName})' )
            exec ( f'{VarName} = wu.setNone ({VarName})' )
            exec (  f'wu.{VarName} = {VarName}' )
            print ( '{:25} set to : {:}'.format (VarName, locals()[VarName]) )
            #exec ( f'del {VarName}' )

#-- Some physical constants
if wu.unDefined ( 'Ra' )           : Ra          = 6366197.7236758135 #-- Earth Radius (m)
if wu.unDefined ( 'Grav' )         : Grav        = 9.81               #-- Gravity (m^2/s
if wu.unDefined ( 'ICE_rho_ice' )  : ICE_rho_ice = 917.0              #-- Ice volumic mass (kg/m3) in LIM3
if wu.unDefined ( 'ICE_rho_sno')   : ICE_rho_sno = 330.0              #-- Snow volumic mass (kg/m3) in LIM3
if wu.unDefined ( 'OCE_rho_liq' )  : OCE_rho_liq = 1026.0             #-- Ocean water volumic mass (kg/m3) in NEMO
if wu.unDefined ( 'ATM_rho' )      : ATM_rho     = 1000.0             #-- Water volumic mass in atmosphere (kg/m^3)
if wu.unDefined ( 'SRF_rho' )      : SRF_rho     = 1000.0             #-- Water volumic mass in surface reservoir (kg/m^3)
if wu.unDefined ( 'RUN_rho' )      : RUN_rho     = 1000.0             #-- Water volumic mass of rivers (kg/m^3)
if wu.unDefined ( 'ICE_rho_pnd' )  : ICE_rho_pnd = 1000.              #-- Water volumic mass in ice ponds (kg/m^3)
if wu.unDefined ( 'YearLength' )   : YearLength  = 365.25 * 24. * 60. * 60. #-- Year length (s) - Use only to compute drif in approximate unit 

# Set libIGCM and machine dependant values
if not 'Files' in config.keys() : config['Files'] = {}

config['Physics'] = { 'Ra':Ra, 'Grav':Grav, 'ICE_rho_ice':ICE_rho_ice, 'ICE_rho_sno':ICE_rho_sno, 'OCE_rho_liq':OCE_rho_liq, 'ATM_rho':ATM_rho, 'SRF_rho':SRF_rho, 'RUN_rho':RUN_rho}

## --------------------------
ICO  = ( 'ICO' in wu.ATM )
LMDZ = ( 'LMD' in wu.ATM )

with open ('SetLibIGCM.py') as f: exec ( f.read() )
config['Files']['TmpDir'] = TmpDir

if libIGCM :
    config['libIGCM'] = {'ARCHIVE':ARCHIVE, 'STORAGE':STORAGE, 'SCRATCHDIR':SCRATCHDIR, 'R_IN':R_IN, 'rebuild':rebuild }

# Import specific module
import nemo, lmdz
## Now import needed scientific modules
import xarray as xr
    
# Output file with water budget diagnostics
if wu.unDefined ( 'FileOut' ) : FileOut = f'CPL_waterbudget_{JobName}_{YearBegin}_{YearEnd}.out'
config['Files']['FileOut'] = FileOut

f_out = open ( FileOut, mode = 'w' )
    
# Useful functions
def kg2Sv    (val, rho=ATM_rho) :
    '''From kg to Sverdrup'''
    return val/dtime_sec*1.0e-6/rho

def kg2myear (val, rho=ATM_rho) :
    '''From kg to m/year'''
    return val/ATM_aire_sea_tot/rho/NbYear

def var2prt (var, small=False, rho=ATM_rho) :
    if small :  return var , kg2Sv(var, rho=rho)*1000., kg2myear(var, rho=rho)*1000.
    else     :  return var , kg2Sv(var, rho=rho)      , kg2myear(var, rho=rho)

def prtFlux (Desc, var, Form='F', small=False, rho=ATM_rho, width=15) :
    if small :
        if Form in ['f', 'F'] : ff=" {:12.4e} kg | {:12.4f} mSv | {:12.4f} mm/year "
        if Form in ['e', 'E'] : ff=" {:12.4e} kg | {:12.4e} mSv | {:12.4e} mm/year "
    else :
        if Form in ['f', 'F'] : ff=" {:12.4e} kg | {:12.4f} Sv  | {:12.4f} m/year  "
        if Form in ['e', 'E'] : ff=" {:12.4e} kg | {:12.4e} Sv  | {:12.4e} m/year  "
    echo ( (' {:>{width}} = ' +ff).format (Desc, *var2prt(var, small, rho=rho), width=width ) )
    return None

def echo (string, end='\n') :
    '''Function to print to stdout *and* output file'''
    print ( str(string), end=end  )
    sys.stdout.flush ()
    f_out.write ( str(string) + end )
    f_out.flush ()
    return None
    
## Set libIGCM directories
if wu.unDefined ('R_OUT'      ) : R_OUT       = os.path.join ( ARCHIVE   , 'IGCM_OUT' )
if wu.unDefined ('R_BUF'      ) : R_BUF       = os.path.join ( SCRATCHDIR, 'IGCM_OUT' )
if wu.unDefined ('L_EXP'      ) : L_EXP       = os.path.join (TagName, SpaceName, ExperimentName, JobName)
if wu.unDefined ('R_SAVE'     ) : R_SAVE      = os.path.join ( R_OUT, L_EXP )
if wu.unDefined ('R_BUFR'     ) : R_BUFR      = os.path.join ( R_BUF, L_EXP )
if wu.unDefined ('POST_DIR'   ) : POST_DIR    = os.path.join ( R_BUFR, 'Out' )
if wu.unDefined ('REBUILD_DIR') : REBUILD_DIR = os.path.join ( R_BUFR, 'REBUILD' )
if wu.unDefined ('R_BUF_KSH'  ) : R_BUF_KSH   = os.path.join ( R_BUFR, 'Out' )
if wu.unDefined ('R_FIGR'     ) : R_FIGR      = os.path.join ( STORAGE, 'IGCM_OUT', L_EXP )

config['libIGCM'] = { 'R_OUT':R_OUT, 'R_BUF':R_BUF, 'L_EXP':L_EXP, 'R_BUFR':R_BUFR, 'POST_DIR':POST_DIR, 'REBUILD_DIR':REBUILD_DIR, 'R_BUF_KSH':R_BUF_KSH, 'R_FIGR':R_FIGR }

# Set directory to extract filesa
if RunDir == None : RunDir = os.path.join ( TmpDir, f'WATER_{JobName}_{YearBegin}_{YearEnd}' )
config['Files']['RunDir'] = RunDir

if not os.path.isdir ( RunDir ) : os.makedirs ( RunDir )

# Set directories to rebuild ocean and ice restart files
RunDirOCE = os.path.join ( RunDir, 'OCE' )
RunDirICE = os.path.join ( RunDir, 'ICE' )
#RunDirATM = os.path.join ( RunDir, 'ATM' )
#RunDirSRF = os.path.join ( RunDir, 'SRF' )
#RunDirRUN = os.path.join ( RunDir, 'RUN' )
#RunDirDYN = os.path.join ( RunDir, 'DYN' )
if not os.path.exists ( RunDirOCE ) : os.mkdir ( RunDirOCE )
if not os.path.exists ( RunDirICE ) : os.mkdir ( RunDirICE )

echo (' ')
echo ( f'JobName   : {JobName}'   )
echo ( f'Comment   : {Comment}'   )
echo ( f'TmpDir    : {TmpDir}'    )
echo ( f'RunDirOCE : {RunDirOCE}' )
echo ( f'RunDirICE : {RunDirICE}' )

echo ( f'\nDealing with {L_EXP}'  )

#-- Model output directories
if Freq == "MO" : FreqDir =  os.path.join ('Output' , 'MO' )
if Freq == "SE" : FreqDir =  os.path.join ('Analyse', 'SE' )
if dir_ATM_his == None :
    dir_ATM_his = os.path.join ( R_SAVE, "ATM", FreqDir )
    config['Files']['dir_ATM_his'] = dir_ATM_his
if dir_SRF_his == None : 
    dir_SRF_his = os.path.join ( R_SAVE, "SRF", FreqDir )
    config['Files']['dir_SRF_his'] = dir_SRF_his
if dir_OCE_his == None :
    dir_OCE_his = os.path.join ( R_SAVE, "OCE", FreqDir )
    config['Files']['dir_OCE_his'] = dir_OCE_his
if dir_ICE_his == None : 
    dir_ICE_his = os.path.join ( R_SAVE, "ICE", FreqDir )
    config['Files']['dir_ICE_his'] = dir_ICE_his

echo ( f'The analysis relies on files from the following model output directories : ' )
echo ( f'{dir_ATM_his}' )
echo ( f'{dir_OCE_his}' )
echo ( f'{dir_ICE_his}' )
echo ( f'{dir_SRF_his}' )

#-- Creates files names
if Period == None :
    if Freq == 'MO' : Period = f'{YearBegin}0101_{YearEnd}1231_1M'
    if Freq == 'SE' : Period = f'SE_{YearBegin}0101_{YearEnd}1231_1M'
    config['Files']['Period'] = Period
if FileCommon == None :
    FileCommon = f'{JobName}_{Period}'
    config['Files']['FileCommon'] = FileCommon

if Title == None :
    Title = f'{JobName} : {Freq} : {YearBegin}-01-01 - {YearEnd}-12-31'
    config['Files']['Title'] = Title

echo ('\nOpen history files' )
if file_ATM_his == None : 
    file_ATM_his = os.path.join (  dir_ATM_his, f'{FileCommon}_histmth.nc' )
    config['Files']['file_ATM_his'] = file_ATM_his
if file_SRF_his == None :
    file_SRF_his = os.path.join (  dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
    config['Files']['file_SRF_his'] = file_SRF_his
#if Routing == 'SECHIBA'  :
#     file_RUN_his = os.path.join (  dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
if Routing == 'SIMPLE' :
    if file_RUN_his == None : 
        file_RUN_his = os.path.join (  dir_SRF_his, f'{FileCommon}_sechiba_history.nc' )
        config['Files']['file_RUN_his'] = file_RUN_his
if file_OCE_his == None :
    file_OCE_his = os.path.join (  dir_OCE_his, f'{FileCommon}_grid_T.nc' )
    config['Files']['file_OCE_his'] = file_OCE_his
if file_OCE_sca == None :
    file_OCE_sca = os.path.join (  dir_OCE_his, f'{FileCommon}_scalar.nc' )
    config['Files']['file_OCE_sca'] = file_OCE_sca
if file_ICE_his == None :
    file_ICE_his = os.path.join (  dir_ICE_his, f'{FileCommon}_icemod.nc' )
    config['Files']['file_ICE_his'] = file_ICE_his
if file_OCE_srf == None :
    file_OCE_srf = os.path.join (  dir_OCE_his, f'{FileCommon}_grid_T.nc' )
    config['Files']['file_OCE_srf'] = file_OCE_srf

d_ATM_his = xr.open_dataset ( file_ATM_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_OCE_his = xr.open_dataset ( file_OCE_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_OCE_sca = xr.open_dataset ( file_OCE_sca, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_ICE_his = xr.open_dataset ( file_ICE_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
if NEMO == '3.6' :d_ICE_his = d_ICE_his.rename ( {'y_grid_T':'y', 'x_grid_T':'x'} )
d_SRF_his = xr.open_dataset ( file_SRF_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
d_OCE_srf = xr.open_dataset ( file_OCE_srf, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()
if Routing == 'SECHIBA' : d_RUN_his = d_SRF_his
if Routing == 'SIMPLE'  : d_RUN_his = xr.open_dataset ( file_RUN_his, use_cftime=True, decode_times=True, decode_cf=True ).squeeze()

    
echo ( f'{file_ATM_his = }' )
echo ( f'{file_SRF_his = }' )
if Routing == 'SIMPLE' : echo ( f'{file_RUN_his = }' )
echo ( f'{file_OCE_his = }' )
echo ( f'{file_ICE_his = }' )
echo ( f'{file_OCE_sca = }' )
echo ( f'{file_OCE_srf = }' )

## Compute run length
dtime = ( d_ATM_his.time_counter_bounds.max() - d_ATM_his.time_counter_bounds.min() )
echo ('\nRun length : {:8.2f} days'.format ( (dtime/np.timedelta64(1, "D")).values ) )
dtime_sec = (dtime/np.timedelta64(1, "s")).values.item() # Convert in seconds

## Compute length of each period
dtime_per = (d_ATM_his.time_counter_bounds[:,-1] -  d_ATM_his.time_counter_bounds[:,0] )
echo ('\nPeriods lengths (days) : {:} days'.format ( (dtime_per/np.timedelta64(1, "D")).values ) )
dtime_per_sec = (dtime_per/np.timedelta64(1, "s")).values # In seconds
dtime_per_sec = xr.DataArray (dtime_per_sec, dims=["time_counter", ], coords=[d_ATM_his.time_counter,] )
dtime_per_sec.attrs['unit'] = 's'

# Number of years
NbYear = dtime_sec / YearLength

#-- Open restart files
YearRes = YearBegin - 1        # Year of the restart of beginning of simulation
YearPre = YearBegin - PackFrequency  # Year to find the tarfile of the restart of beginning of simulation

config['Files']['YearPre'] = f'{YearBegin}'

echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')

if TarRestartPeriod_beg == None : 
    echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')
    TarRestartPeriod_beg = f'{YearPre}0101_{YearRes}1231'
    config['Files']['TarRestartPeriod_beg'] = TarRestartPeriod_beg

if TarRestartPeriod_end == None : 
    YearPre = YearBegin - PackFrequency  # Year to find the tarfile of the restart of beginning of simulation
    echo (f'Restart dates - Start : {YearRes}-12-31  /  End : {YearEnd}-12-31 ')
    TarRestartPeriod_end = f'{YearBegin}0101_{YearEnd}1231'
    config['Files']['TarRestartPeriod_end'] = TarRestartPeriod_end

if tar_restart_beg == None :
    tar_restart_beg = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{TarRestartPeriod_beg}_restart.tar' )
    config['Files']['tar_restart_beg'] = tar_restart_beg
if tar_restart_end == None :
    tar_restart_end = os.path.join ( R_SAVE, 'RESTART', f'{JobName}_{TarRestartPeriod_end}_restart.tar' )
    config['Files']['tar_restart_end'] = tar_restart_end

echo ( f'{tar_restart_beg}' )
echo ( f'{tar_restart_end}' )

if file_ATM_beg == None : 
    file_ATM_beg = f'{RunDir}/ATM_{JobName}_{YearRes}1231_restartphy.nc'
    config['Files']['file_ATM_beg'] = file_ATM_beg
if file_ATM_end == None :
    file_ATM_end = f'{RunDir}/ATM_{JobName}_{YearEnd}1231_restartphy.nc'
    config['Files']['file_ATM_end'] = file_ATM_end

if file_DYN_beg == None :
    if LMDZ : file_DYN_beg = f'{RunDir}/ATM_{JobName}_{YearRes}1231_restart.nc'
    if ICO  : file_DYN_beg = f'{RunDir}/ICO_{JobName}_{YearRes}1231_restart.nc'
    config['Files']['file_DYN_beg'] = file_DYN_beg
    
if file_DYN_end == None : 
    if LMDZ : file_DYN_end = f'{RunDir}/ATM_{JobName}_{YearEnd}1231_restart.nc'
    if ICO  : file_DYN_end = f'{RunDir}/ICO_{JobName}_{YearEnd}1231_restart.nc'
    config['Files']['file_DYN_end'] = file_DYN_end
    
if file_SRF_beg == None :
    file_SRF_beg = f'{RunDir}/SRF_{JobName}_{YearRes}1231_sechiba_rest.nc'
    config['Files']['file_SRF_beg'] = file_SRF_beg
if file_SRF_end == None :
    file_SRF_end = f'{RunDir}/SRF_{JobName}_{YearEnd}1231_sechiba_rest.nc'
    config['Files']['file_SRF_end'] = file_SRF_end
    
if file_OCE_beg == None : 
    file_OCE_beg = f'{RunDir}/OCE_{JobName}_{YearRes}1231_restart.nc'
    config['Files']['file_OCE_beg'] = file_OCE_beg
if file_OCE_end == None :
    file_OCE_end = f'{RunDir}/OCE_{JobName}_{YearEnd}1231_restart.nc'
    config['Files']['file_OCE_end'] = file_OCE_end
if file_ICE_beg == None :
    file_ICE_beg = f'{RunDir}/ICE_{JobName}_{YearRes}1231_restart_icemod.nc'
    config['Files']['file_ICE_beg'] = file_ICE_beg
if file_ICE_end == None : 
    file_ICE_end = f'{RunDir}/ICE_{JobName}_{YearEnd}1231_restart_icemod.nc'
    config['Files']['file_ICE_end'] = file_ICE_end

liste_beg = [file_ATM_beg, file_DYN_beg, file_SRF_beg]
liste_end = [file_ATM_end, file_DYN_end, file_SRF_end]

if Routing == 'SIMPLE' :
    if file_RUN_beg == None : 
        file_RUN_beg = f'{RunDir}/SRF_{JobName}_{YearRes}1231_routing_restart.nc'
        config['Files']['file_RUN_beg'] = file_RUN_beg
    if file_RUN_end == None : 
        file_RUN_end = f'{RunDir}/SRF_{JobName}_{YearEnd}1231_routing_restart.nc'
        config['Files']['file_RUN_end'] = file_RUN_end
        
    liste_beg.append ( file_RUN_beg )
    liste_end.append ( file_RUN_end )
    echo ( f'{file_RUN_beg = }' )
    echo ( f'{file_RUN_end = }' )

echo ( f'{file_ATM_beg = }' )
echo ( f'{file_ATM_end = }' )
echo ( f'{file_DYN_beg = }' )
echo ( f'{file_DYN_end = }' )
echo ( f'{file_SRF_beg = }' )
echo ( f'{file_SRF_end = }' )
echo ( f'{file_RUN_beg = }' )
echo ( f'{file_RUN_end = }' )
echo ( f'{file_OCE_beg = }' )
echo ( f'{file_OCE_end = }' )
echo ( f'{file_ICE_beg = }' )
echo ( f'{file_ICE_end = }' )

echo ('\nExtract restart files from tar : ATM, ICO and SRF')
for resFile in liste_beg + liste_end :
    if os.path.exists ( os.path.join (RunDir, resFile) ) :
        echo ( f'file found : {resFile = }' )
    else :
        base_file = Path (file_name).stem # basename, and remove suffix
        command =  f'cd {RunDir} ; tar xf {tar_restart_beg} {base_resFile}.nc'
        echo ( command )
        ierr = os.system ( command )
        if ierr == 0 : echo ( f'tar done : {base_resFile}')
        else         : raise Exception ( f'command failed : {command}' )

echo ('\nOpening ATM SRF and ICO restart files')
d_ATM_beg = xr.open_dataset ( os.path.join (RunDir, file_ATM_beg), decode_times=False, decode_cf=True).squeeze()
d_ATM_end = xr.open_dataset ( os.path.join (RunDir, file_ATM_end), decode_times=False, decode_cf=True).squeeze()
d_SRF_beg = xr.open_dataset ( os.path.join (RunDir, file_SRF_beg), decode_times=False, decode_cf=True).squeeze()
d_SRF_end = xr.open_dataset ( os.path.join (RunDir, file_SRF_end), decode_times=False, decode_cf=True).squeeze()
d_DYN_beg = xr.open_dataset ( os.path.join (RunDir, file_DYN_beg), decode_times=False, decode_cf=True).squeeze()
d_DYN_end = xr.open_dataset ( os.path.join (RunDir, file_DYN_end), decode_times=False, decode_cf=True).squeeze()

echo ('\nExtract and rebuild OCE and ICE restarts')
def get_ndomain (zfile) :
    #d_zfile = xr.open_dataset (zfile, decode_times=False).squeeze()
    #ndomain_opa = d_zfile.attrs['DOMAIN_number_total']
    #d_zfile.close ()
    ndomain_opa = subprocess.getoutput ( f'ls {zfile}_*.nc | wc -l' ) #.format()
    return int (ndomain_opa)

def extract_and_rebuild ( file_name=file_OCE_beg, tar_restart=tar_restart_end, RunDirComp=RunDirOCE ) :
    '''Extract restart file from tar. Rebuild ocean files if needed'''
    echo ( f'file to extract : {file_name} ' )
    if os.path.exists ( file_name ) :
        echo ( f'-- File ready : {file_name}' )
    else : 
        echo ( f'-- Extracting {file_name}' )
        base_resFile = Path (file_name).stem # basename, and remove suffix
        # Try to extract the rebuilded file
        command =  f'cd {RunDirComp} ; tar xf {tar_restart} {base_resFile}.nc'
        echo ( command )
        ierr = os.system ( command )
        if ierr == 0 :
            echo ( f'tar done : {base_resFile}')
            command = f'cd {RunDirComp} ; mv {base_resFile}.nc {RunDir}'
            ierr = os.system ( command )
            if ierr == 0 : echo ( f'command done : {command}' )
            else         : raise Exception ( f'command failed : {command}' )
        else :
            if not os.path.exists ( os.path.join (RunDir, f'{base_resFile}_0000.nc') ):
                command =  f'cd {RunDirComp} ; tar xf {tar_restart_end} {base_resFile}_*.nc'
                echo ( command )
                ierr = os.system ( command )
                if ierr == 0 : echo ( f'tar done : {file_OCE_beg}')
                else         : raise Exception ( f'command failed : {command}' )
                echo ('extract ndomain' )
            ndomain_opa = get_ndomain ( os.path.join (RunDir, f'{base_resFile}') )
            command = f'cd {RunDirComp} ; {rebuild} {base_resFile} {ndomain_opa} ; mv {base_resFile}.nc {RunDir}'
            echo ( command )
            ierr = os.system ( command )
            if ierr == 0 : echo ( f'Rebuild done : {base_resFile}.nc')
            else         : raise Exception ( f'command failed : {command}' )

extract_and_rebuild ( file_name=file_OCE_beg, tar_restart=tar_restart_beg, RunDirComp=RunDirOCE )
extract_and_rebuild ( file_name=file_OCE_end, tar_restart=tar_restart_end, RunDirComp=RunDirOCE )
extract_and_rebuild ( file_name=file_ICE_beg, tar_restart=tar_restart_beg, RunDirComp=RunDirICE )
extract_and_rebuild ( file_name=file_ICE_end, tar_restart=tar_restart_end, RunDirComp=RunDirICE )

echo ('Opening OCE and ICE restart files')
if NEMO == 3.6 : 
    d_OCE_beg = xr.open_dataset ( os.path.join (RunDir, file_OCE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_OCE_end = xr.open_dataset ( os.path.join (RunDir, file_OCE_end), decode_times=False, decode_cf=True).squeeze()
    d_ICE_beg = xr.open_dataset ( os.path.join (RunDir, file_ICE_beg), decode_times=False, decode_cf=True).squeeze()
    d_ICE_end = xr.open_dataset ( os.path.join (RunDir, file_ICE_end), decode_times=False, decode_cf=True).squeeze()
if NEMO == 4.0 or NEMO == 4.2 : 
    d_OCE_beg = xr.open_dataset ( os.path.join (RunDir, file_OCE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_OCE_end = xr.open_dataset ( os.path.join (RunDir, file_OCE_end), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_ICE_beg = xr.open_dataset ( os.path.join (RunDir, file_ICE_beg), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()
    d_ICE_end = xr.open_dataset ( os.path.join (RunDir, file_ICE_end), decode_times=False, decode_cf=True, drop_variables=['y', 'x']).squeeze()

## Write the full configuration
config_out = open (FullIniFile, 'w')
config.write (config_out )
config_out.close ()

for var in d_SRF_beg.variables :
    d_SRF_beg[var] = d_SRF_beg[var].where (  d_SRF_beg[var]<1.e20, 0.)
    d_SRF_end[var] = d_SRF_end[var].where (  d_SRF_end[var]<1.e20, 0.)

if ICO :
    d_RUN_beg = xr.open_dataset ( os.path.join (RunDir, file_RUN_beg), decode_times=False, decode_cf=True).squeeze()
    d_RUN_end = xr.open_dataset ( os.path.join (RunDir, file_RUN_end), decode_times=False, decode_cf=True).squeeze()

def kg2Sv  (val, rho=OCE_rho_liq) :
    '''From kg to Sverdrup'''
    return val/dtime_sec*1.0e-6/rho

def kg2myear (val, rho=OCE_rho_liq) :
    '''From kg to m/year'''
    return val/OCE_aire_tot/rho/NbYear

def var2prt (var, small=False) :
    if small :  return var , kg2Sv(var)*1000., kg2myear(var)*1000.
    else     :  return var , kg2Sv(var)      , kg2myear(var)

def prtFlux (Desc, var, Form='F', small=False) :
    if small :
        if Form in ['f', 'F'] : ff=" {:12.4e} kg | {:12.4f} mSv | {:12.4f} mm/year "
        if Form in ['e', 'E'] : ff=" {:12.4e} kg | {:12.4e} mSv | {:12.4e} mm/year "
    else :
        if Form in ['f', 'F'] : ff=" {:12.4e} kg | {:12.4f} Sv  | {:12.4f} m/year  "
        if Form in ['e', 'E'] : ff=" {:12.4e} kg | {:12.4e} Sv  | {:12.4e} m/year  "
    echo ( (' {:>15} = ' +ff).format (Desc, *var2prt(var, small) ) )
    return None



# ATM grid with cell surfaces
if ICO :
    if ATM_HIS == 'latlon' :
        jpja, jpia = d_ATM_his['aire'][0].shape   
        file_ATM_aire = os.path.join ( R_IN, 'ATM', 'GRID', f'aire_{ATM}_to_{jpia}x{jpja}.nc' )
        config['Files']['file_ATM_aire'] = file_ATM_aire
        echo ( f'Aire sur grille reguliere : {file_ATM_aire = }' )
        d_ATM_aire = xr.open_dataset ( file_ATM_aire, decode_times=False ).squeeze()
        ATM_aire = lmdz.geo2point ( d_ATM_aire ['aire'].squeeze(), cumulPoles=True )
        ATM_fter = lmdz.geo2point ( d_ATM_his ['fract_ter'][0] )
        ATM_foce = lmdz.geo2point ( d_ATM_his ['fract_oce'][0] )
        ATM_fsic = lmdz.geo2point ( d_ATM_his ['fract_sic'][0] )
        ATM_flic = lmdz.geo2point ( d_ATM_his ['fract_lic'][0] )
        SRF_aire = lmdz.geo2point ( d_SRF_his ['Areas'] * d_SRF_his ['Contfrac'] )
        #SRF_aire = ATM_aire * lmdz.geo2point (d_SRF_his ['Contfrac'] )
        #SRF_aire = ATM_aire * ATM_fter
    if ATM_HIS == 'ico' :
        echo ( f'Aire sur grille icosaedre : {file_ATM_aire = }' )
        
if LMDZ :
    ATM_aire = lmdz.geo2point ( d_ATM_his ['aire'][0], cumulPoles=True )
    ATM_fter = lmdz.geo2point ( d_ATM_his ['fract_ter'][0] )
    ATM_foce = lmdz.geo2point ( d_ATM_his ['fract_oce'][0] )
    ATM_fsic = lmdz.geo2point ( d_ATM_his ['fract_sic'][0] )
    ATM_flic = lmdz.geo2point ( d_ATM_his ['fract_lic'][0] )
    #SRF_aire = lmdz.geo2point ( d_SRF_his['Areas'] * d_SRF_his['Contfrac'] )
    SRF_aire = ATM_aire * lmdz.geo2point ( d_SRF_his['Contfrac'] )
    
ATM_fsea      = ATM_foce + ATM_fsic
ATM_flnd      = ATM_fter + ATM_flic
ATM_aire_fter = ATM_aire * ATM_fter
ATM_aire_flic = ATM_aire * ATM_flic
ATM_aire_fsic = ATM_aire * ATM_fsic
ATM_aire_foce = ATM_aire * ATM_foce
ATM_aire_flnd = ATM_aire * ATM_flnd
ATM_aire_fsea = ATM_aire * ATM_fsea

SRF_aire = SRF_aire.where ( SRF_aire < 1E15, 0.)

if ICO :
    # Area on icosahedron grid
    file_DYN_aire = os.path.join ( R_IN, 'ATM', 'GRID', ATM+'_grid.nc' )
    d_DYN_aire = xr.open_dataset ( file_DYN_aire, decode_times=False).squeeze()
    d_DYN_aire = d_DYN_aire.rename ( {'cell':'cell_mesh'} )
    DYN_aire   = d_DYN_aire['aire']

    DYN_fsea = d_DYN_aire ['fract_oce'] + d_DYN_aire ['fract_sic']
    DYN_flnd = 1.0 - DYN_fsea
    
if LMDZ :
    # Area on lon/lat grid
    DYN_aire = ATM_aire
    DYN_fsea = ATM_fsea
    DYN_flnd = ATM_flnd
    DYN_fter = d_ATM_beg['FTER']
    DYN_flic = d_ATM_beg['FTER']

def ATM_stock_int (stock) :
    '''Integrate (* surface) stock on atmosphere grid'''
    ATM_stock_int  = wu.Psum ( (stock * DYN_aire).to_masked_array().ravel() ) 
    return ATM_stock_int

def ATM_flux_int (flux) :
    '''Integrate (* time * surface) flux on atmosphere grid'''
    ATM_flux_int  = wu.Psum ( (flux * dtime_per_sec * ATM_aire).to_masked_array().ravel() )
    return ATM_flux_int

def SRF_stock_int (stock) :
    '''Integrate (* surface) stock on land grid'''
    ATM_stock_int  = wu.Ksum (  ( (stock * DYN_aire_fter).to_masked_array().ravel()) )
    return ATM_stock_int

def SRF_flux_int (flux) :
    '''Integrate (* time * surface) flux on land grid'''
    SRF_flux_int  = wu.Psum (  (flux * dtime_per_sec * SRF_aire).to_masked_array().ravel() )
    return SRF_flux_int

def OCE_stock_int (stock) :
    '''Integrate stock on ocean grid'''
    OCE_stock_int = np.sum (  np.sort ( (stock * OCE_aire ).to_masked_array().ravel()) )
    return OCE_stock_int

def ONE_stock_int (stock) :
    '''Sum stock'''
    ONE_stock_int = np.sum (  np.sort ( (stock ).to_masked_array().ravel()) )
    return ONE_stock_int

def OCE_flux_int (flux) :
    '''Integrate flux on oce grid'''
    OCE_flux_int = np.sum (  np.sort ( (flux * OCE_aire * dtime_per_sec).to_masked_array().ravel()) )
    return OCE_flux_int

def ONE_flux_int (flux) :
    '''Integrate flux on oce grid'''
    OCE_flux_int = np.sum (  np.sort ( (flux * dtime_per_sec).to_masked_array().ravel()) )
    return OCE_flux_int
    
#if LMDZ :
#    d_ATM_beg = d_ATM_beg.assign ( coords={'lon':d_ATM_beg.lon*180./np.pi} )

# Get mask and surfaces
sos = d_OCE_his ['sos'][0].squeeze()
OCE_msk = nemo.lbc_mask ( xr.where ( sos>0, 1., 0.0 ), cd_type = 'T' )
so = sos = d_OCE_his ['sos'][0].squeeze()
OCE_msk3 = nemo.lbc_mask ( xr.where ( so>0., 1., 0. ), cd_type = 'T', sval = 0. )

# lbc_mask removes the duplicate points (periodicity and north fold)
OCE_aire = nemo.lbc_mask ( d_OCE_his ['area'] * OCE_msk, cd_type = 'T', sval = 0.0 )
ICE_aire = OCE_aire

ATM_aire_tot = ONE_stock_int (ATM_aire)
SRF_aire_tot = ONE_stock_int (SRF_aire)
OCE_aire_tot = ONE_stock_int (OCE_aire)
ICE_aire_tot = ONE_stock_int (ICE_aire)

ATM_aire_sea     = ATM_aire * ATM_fsea
ATM_aire_sea_tot = ONE_stock_int ( ATM_aire_sea )

echo ( '\n====================================================================================' )
echo ( '-- NEMO change in stores (for the records)' )
#-- Note that the total number of days of the run should be diagnosed rather than assumed
#-- Here the result is in Sv
#
#-- Change in ocean volume in freshwater equivalent

OCE_ssh_beg = d_OCE_beg['sshn']
OCE_ssh_end = d_OCE_end['sshn']
OCE_sum_ssh_beg = OCE_stock_int ( OCE_ssh_beg )
OCE_sum_ssh_end = OCE_stock_int ( OCE_ssh_end )

OCE_mas_wat_beg = OCE_sum_ssh_beg * OCE_rho_liq
OCE_mas_wat_end = OCE_sum_ssh_end * OCE_rho_liq

echo ( 'OCE_sum_ssh_beg = {:12.6e} m^3 | OCE_sum_ssh_end = {:12.6e} m^3'.format (OCE_sum_ssh_beg, OCE_sum_ssh_end) )
dOCE_vol_liq = ( OCE_sum_ssh_end - OCE_sum_ssh_beg )
dOCE_mas_liq = dOCE_vol_liq * OCE_rho_liq
dOCE_mas_wat = dOCE_mas_liq

echo ( 'dOCE vol    = {:12.3e} m^3'.format (dOCE_vol_liq) )
echo ( 'dOCE ssh    = {:12.3e} m  '.format (dOCE_vol_liq/OCE_aire_tot) )
echo ( 'dOCE mass   = {:12.3e} kg '.format (dOCE_mas_liq) )
echo ( 'dOCE mass   = {:12.3e} Sv '.format (dOCE_mas_liq/dtime_sec*1E-6/OCE_rho_liq) )

## Glace et neige
if NEMO == 3.6 :
    ICE_ice_beg = d_ICE_beg['v_i_htc1']+d_ICE_beg['v_i_htc2']+d_ICE_beg['v_i_htc3']+d_ICE_beg['v_i_htc4']+d_ICE_beg['v_i_htc5']
    ICE_ice_end = d_ICE_end['v_i_htc1']+d_ICE_end['v_i_htc2']+d_ICE_end['v_i_htc3']+d_ICE_end['v_i_htc4']+d_ICE_end['v_i_htc5']

    ICE_sno_beg = d_ICE_beg['v_s_htc1']+d_ICE_beg['v_s_htc2']+d_ICE_beg['v_s_htc3']+d_ICE_beg['v_s_htc4']+d_ICE_beg['v_s_htc5']
    ICE_sno_end = d_ICE_end['v_s_htc1']+d_ICE_end['v_s_htc2']+d_ICE_end['v_s_htc3']+d_ICE_end['v_s_htc4']+d_ICE_end['v_s_htc5']
    
    ICE_pnd_beg = 0.0 ; ICE_pnd_end = 0.0
    ICE_fzl_beg = 0.0 ; ICE_fzl_end = 0.0

    ICE_mas_wat_beg = OCE_stock_int ( (ICE_ice_beg*ICE_rho_ice + ICE_sno_beg*ICE_rho_sno) )
    ICE_mas_wat_end = OCE_stock_int ( (ICE_ice_end*ICE_rho_ice + ICE_sno_end*ICE_rho_sno) )
    
if NEMO == 4.0 or NEMO == 4.2 :
    ICE_ice_beg = d_ICE_beg ['v_i']  ; ICE_ice_end = d_ICE_end ['v_i']
    ICE_sno_beg = d_ICE_beg ['v_s']  ; ICE_sno_end = d_ICE_end ['v_s']
    ICE_pnd_beg = d_ICE_beg ['v_ip'] ; ICE_pnd_end = d_ICE_end ['v_ip']
    ICE_fzl_beg = d_ICE_beg ['v_il'] ; ICE_fzl_end = d_ICE_end ['v_il']
    
    ICE_mas_wat_beg = OCE_stock_int ( d_ICE_beg['snwice_mass'] )
    ICE_mas_wat_end = OCE_stock_int ( d_ICE_end['snwice_mass'] )
    
    
ICE_vol_ice_beg = OCE_stock_int ( ICE_ice_beg )
ICE_vol_ice_end = OCE_stock_int ( ICE_ice_end )

ICE_vol_sno_beg = OCE_stock_int ( ICE_sno_beg )
ICE_vol_sno_end = OCE_stock_int ( ICE_sno_end )

ICE_vol_pnd_beg = OCE_stock_int ( ICE_pnd_beg )
ICE_vol_pnd_end = OCE_stock_int ( ICE_pnd_end )

ICE_vol_fzl_beg = OCE_stock_int ( ICE_fzl_beg )
ICE_vol_fzl_end = OCE_stock_int ( ICE_fzl_end )

#-- Converting to freswater volume
dICE_vol_ice = ICE_vol_ice_end - ICE_vol_ice_beg
dICE_mas_ice = dICE_vol_ice * ICE_rho_ice

dICE_vol_sno = ICE_vol_sno_end - ICE_vol_sno_beg
dICE_mas_sno = dICE_vol_sno * ICE_rho_sno

dICE_vol_pnd = ICE_vol_pnd_end - ICE_vol_pnd_beg
dICE_mas_pnd = dICE_vol_pnd * ICE_rho_pnd

dICE_vol_fzl= ICE_vol_fzl_end - ICE_vol_fzl_beg
dICE_mas_fzl = dICE_vol_fzl * ICE_rho_pnd

if NEMO == 3.6 :
    dICE_mas_wat = dICE_mas_ice + dICE_mas_sno
    dSEA_mas_wat = dOCE_mas_wat + dICE_mas_ice + dICE_mas_sno

if NEMO == 4.0 or NEMO == 4.2 :
    dICE_mas_wat = ICE_mas_wat_end - ICE_mas_wat_beg
    dSEA_mas_wat = dOCE_mas_wat + dICE_mas_wat

echo ( f'ICE_vol_ice_beg = {ICE_vol_ice_beg:12.6e} m^3 | ICE_vol_ice_end = {ICE_vol_ice_end:12.6e} m^3' )
echo ( f'ICE_vol_sno_beg = {ICE_vol_sno_beg:12.6e} m^3 | ICE_vol_sno_end = {ICE_vol_sno_end:12.6e} m^3' )
echo ( f'ICE_vol_pnd_beg = {ICE_vol_pnd_beg:12.6e} m^3 | ICE_vol_pnd_end = {ICE_vol_pnd_end:12.6e} m^3' )
echo ( f'ICE_vol_fzl_beg = {ICE_vol_fzl_beg:12.6e} m^3 | ICE_vol_fzl_end = {ICE_vol_fzl_end:12.6e} m^3' )

echo ( f'dICE_vol_ice   = {dICE_vol_ice:12.3e} m^3' )
echo ( f'dICE_vol_sno   = {dICE_vol_sno:12.3e} m^3' )
echo ( f'dICE_vol_pnd   = {dICE_vol_pnd:12.3e} m^3' )
echo ( f'dICE_mas_ice   = {dICE_mas_ice:12.3e} m^3' )
echo ( f'dICE_mas_sno   = {dICE_mas_sno:12.3e} m^3' )  
echo ( f'dICE_mas_pnd   = {dICE_mas_pnd:12.3e} m^3' )  
echo ( f'dICE_mas_fzl   = {dICE_mas_fzl:12.3e} m^3' )  
echo ( f'dICE_mas_wat   = {dICE_mas_wat:12.3e} m^3' ) 


SEA_mas_wat_beg = OCE_mas_wat_beg + ICE_mas_wat_beg
SEA_mas_wat_end = OCE_mas_wat_end + ICE_mas_wat_end

echo ( '\n------------------------------------------------------------')
echo ( 'Variation du contenu en eau ocean + glace ' )
echo ( 'dMass (ocean)   = {:12.6e} kg '.format(dSEA_mas_wat) )
echo ( 'dVol  (ocean)   = {:12.3e} Sv '.format(dSEA_mas_wat/dtime_sec*1E-6/OCE_rho_liq) )
echo ( 'dVol  (ocean)   = {:12.3e} m  '.format(dSEA_mas_wat*1E-3/OCE_aire_tot) )


echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- ATM changes in stores ' )

#-- Change in precipitable water from the atmosphere daily and monthly files
#-- Compute sum weighted by gridcell area (kg/m2) then convert to Sv

# ATM vertical grid
ATM_Ahyb = d_ATM_his['Ahyb'].squeeze()
ATM_Bhyb = d_ATM_his['Bhyb'].squeeze()
klevp1 = ATM_Ahyb.shape[0]

# Surface pressure
if ICO : 
    DYN_ps_beg = d_DYN_beg['ps']
    DYN_ps_end = d_DYN_end['ps']
    
if LMDZ : 
    DYN_ps_beg =  lmdz.geo2point ( d_DYN_beg['ps'].isel(rlonv=slice(0,-1)) )
    DYN_ps_end =  lmdz.geo2point ( d_DYN_end['ps'].isel(rlonv=slice(0,-1)) )
    
# 3D Pressure
DYN_p_beg = ATM_Ahyb + ATM_Bhyb * DYN_ps_beg
DYN_p_end = ATM_Ahyb + ATM_Bhyb * DYN_ps_end

# Layer thickness
DYN_sigma_beg = DYN_p_beg[0:-1]*0.
DYN_sigma_end = DYN_p_end[0:-1]*0. 

for k in np.arange (klevp1-1) :
    DYN_sigma_beg[k,:] = (DYN_p_beg[k,:] - DYN_p_beg[k+1,:]) / Grav
    DYN_sigma_end[k,:] = (DYN_p_end[k,:] - DYN_p_end[k+1,:]) / Grav

DYN_sigma_beg = DYN_sigma_beg.rename ( {'klevp1':'sigs'} )
DYN_sigma_end = DYN_sigma_end.rename ( {'klevp1':'sigs'} )

##-- Vertical and horizontal integral, and sum of liquid, solid and vapor water phases
if LMDZ :
    try : 
        DYN_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2Ov'] + d_DYN_beg['H2Ol'] + d_DYN_beg['H2Oi']).isel(rlonv=slice(0,-1) ) )
        DYN_wat_end = lmdz.geo3point ( (d_DYN_end['H2Ov'] + d_DYN_end['H2Ol'] + d_DYN_end['H2Oi']).isel(rlonv=slice(0,-1) ) )
    except :
        DYN_wat_beg = lmdz.geo3point ( (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).isel(rlonv=slice(0,-1) ) )
        DYN_wat_end = lmdz.geo3point ( (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).isel(rlonv=slice(0,-1) ) )
if ICO :
    try :
        DYN_wat_beg = (d_DYN_beg['H2O_g'] + d_DYN_beg['H2O_l'] + d_DYN_beg['H2O_s']).rename ( {'lev':'sigs'} )
        DYN_wat_end = (d_DYN_end['H2O_g'] + d_DYN_end['H2O_l'] + d_DYN_end['H2O_s']).rename ( {'lev':'sigs'} )
    except : 
        DYN_wat_beg = (d_DYN_beg['q'].isel(nq=0) + d_DYN_beg['q'].isel(nq=1) + d_DYN_beg['q'].isel(nq=2) ).rename ( {'lev':'sigs'} )
        DYN_wat_end = (d_DYN_end['q'].isel(nq=0) + d_DYN_end['q'].isel(nq=1) + d_DYN_end['q'].isel(nq=2) ).rename ( {'lev':'sigs'} )
    
DYN_mas_wat_beg = ATM_stock_int (DYN_sigma_beg * DYN_wat_beg)
DYN_mas_wat_end = ATM_stock_int (DYN_sigma_end * DYN_wat_end)

dDYN_mas_wat = DYN_mas_wat_end - DYN_mas_wat_beg

echo ( '\nVariation du contenu en eau atmosphere (dynamique) ' )
echo ( 'DYN_mas_beg = {:12.6e} kg | DYN_mas_end = {:12.6e} kg'.format (DYN_mas_wat_beg, DYN_mas_wat_end) )
echo ( 'dMass (atm)   = {:12.3e} kg '.format (dDYN_mas_wat) )
echo ( 'dMass (atm)   = {:12.3e} Sv '.format (dDYN_mas_wat/dtime_sec*1.e-6/ATM_rho) )
echo ( 'dMass (atm)   = {:12.3e} m  '.format (dDYN_mas_wat/ATM_aire_sea_tot/ATM_rho) )

ATM_sno_beg = d_ATM_beg['SNOW01']*d_ATM_beg['FTER']+d_ATM_beg['SNOW02']*d_ATM_beg['FLIC']+d_ATM_beg['SNOW03']*d_ATM_beg['FOCE']+d_ATM_beg['SNOW04']*d_ATM_beg['FSIC']
ATM_sno_end = d_ATM_end['SNOW01']*d_ATM_end['FTER']+d_ATM_end['SNOW02']*d_ATM_end['FLIC']+d_ATM_end['SNOW03']*d_ATM_end['FOCE']+d_ATM_end['SNOW04']*d_ATM_end['FSIC']

ATM_qs_beg  = d_ATM_beg['QS01']*d_ATM_beg['FTER']+d_ATM_beg['QS02']*d_ATM_beg['FLIC']+d_ATM_beg['QS03']*d_ATM_beg['FOCE']+d_ATM_beg['QS04']*d_ATM_beg['FSIC']
ATM_qs_end  = d_ATM_end['QS01']*d_ATM_end['FTER']+d_ATM_end['QS02']*d_ATM_end['FLIC']+d_ATM_end['QS03']*d_ATM_end['FOCE']+d_ATM_end['QS04']*d_ATM_end['FSIC']

ATM_qsol_beg = d_ATM_beg['QSOL']
ATM_qsol_end = d_ATM_end['QSOL']

ATM_qs01_beg  = d_ATM_beg['QS01'] * d_ATM_beg['FTER']
ATM_qs01_end  = d_ATM_end['QS01'] * d_ATM_end['FTER']
ATM_qs02_beg  = d_ATM_beg['QS02'] * d_ATM_beg['FLIC']
ATM_qs02_end  = d_ATM_end['QS02'] * d_ATM_end['FLIC']
ATM_qs03_beg  = d_ATM_beg['QS03'] * d_ATM_beg['FOCE']
ATM_qs03_end  = d_ATM_end['QS03'] * d_ATM_end['FOCE']
ATM_qs04_beg  = d_ATM_beg['QS04'] * d_ATM_beg['FSIC']
ATM_qs04_end  = d_ATM_end['QS04'] * d_ATM_end['FSIC']  

if ICO :
   ATM_sno_beg   = ATM_sno_beg .rename ( {'points_physiques':'cell_mesh'} )
   ATM_sno_end   = ATM_sno_end .rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs_beg    = ATM_qs_beg  .rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs_end    = ATM_qs_end  .rename ( {'points_physiques':'cell_mesh'} )
   ATM_qsol_beg  = ATM_qsol_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qsol_end  = ATM_qsol_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs01_beg  = ATM_qs01_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs01_end  = ATM_qs01_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs02_beg  = ATM_qs02_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs02_end  = ATM_qs02_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs03_beg  = ATM_qs03_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs03_end  = ATM_qs03_end.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs04_beg  = ATM_qs04_beg.rename ( {'points_physiques':'cell_mesh'} )
   ATM_qs04_end  = ATM_qs04_end.rename ( {'points_physiques':'cell_mesh'} ) 

echo ('Computing atmosphere integrals')
ATM_mas_sno_beg   = ATM_stock_int ( ATM_sno_beg  )
ATM_mas_sno_end   = ATM_stock_int ( ATM_sno_end  )
ATM_mas_qs_beg    = ATM_stock_int ( ATM_qs_beg   )
ATM_mas_qs_end    = ATM_stock_int ( ATM_qs_end   )
ATM_mas_qsol_beg  = ATM_stock_int ( ATM_qsol_beg )
ATM_mas_qsol_end  = ATM_stock_int ( ATM_qsol_end )
ATM_mas_qs01_beg  = ATM_stock_int ( ATM_qs01_beg )
ATM_mas_qs01_end  = ATM_stock_int ( ATM_qs01_end )
ATM_mas_qs02_beg  = ATM_stock_int ( ATM_qs02_beg )
ATM_mas_qs02_end  = ATM_stock_int ( ATM_qs02_end )
ATM_mas_qs03_beg  = ATM_stock_int ( ATM_qs03_beg )
ATM_mas_qs03_end  = ATM_stock_int ( ATM_qs03_end )
ATM_mas_qs04_beg  = ATM_stock_int ( ATM_qs04_beg )
ATM_mas_qs04_end  = ATM_stock_int ( ATM_qs04_end )

dATM_mas_sno  = ATM_mas_sno_end  - ATM_mas_sno_beg
dATM_mas_qs   = ATM_mas_qs_end   - ATM_mas_qs_beg
dATM_mas_qsol = ATM_mas_qsol_end - ATM_mas_qsol_beg

dATM_mas_qs01 = ATM_mas_qs01_end - ATM_mas_qs01_beg
dATM_mas_qs02 = ATM_mas_qs02_end - ATM_mas_qs02_beg
dATM_mas_qs03 = ATM_mas_qs03_end - ATM_mas_qs03_beg
dATM_mas_qs04 = ATM_mas_qs04_end - ATM_mas_qs04_beg

echo ( '\nVariation du contenu en neige atmosphere (calottes)' )
echo ( 'ATM_mas_sno_beg  = {:12.6e} kg | ATM_mas_sno_end  = {:12.6e} kg'.format (ATM_mas_sno_beg, ATM_mas_sno_end) )
echo ( 'dMass (neige atm) = {:12.3e} kg '.format (dATM_mas_sno ) )
echo ( 'dMass (neige atm) = {:12.3e} Sv '.format (dATM_mas_sno/dtime_sec*1e-6/ICE_rho_ice) )
echo ( 'dMass (neige atm) = {:12.3e} m  '.format (dATM_mas_sno/ATM_aire_sea_tot/ATM_rho) )

echo ( '\nVariation du contenu humidite du sol' )
echo ( 'ATM_mas_qs_beg  = {:12.6e} kg | ATM_mas_qs_end  = {:12.6e} kg'.format (ATM_mas_qs_beg, ATM_mas_qs_end) )
echo ( 'dMass (neige atm) = {:12.3e} kg '.format (dATM_mas_qs ) )
echo ( 'dMass (neige atm) = {:12.3e} Sv '.format (dATM_mas_qs/dtime_sec*1e-6/ATM_rho) )
echo ( 'dMass (neige atm) = {:12.3e} m  '.format (dATM_mas_qs/ATM_aire_sea_tot/ATM_rho) )

echo ( '\nVariation du contenu en eau+neige atmosphere ' )
echo ( 'dMass (eau + neige atm) = {:12.3e} kg '.format (  dDYN_mas_wat + dATM_mas_sno) )
echo ( 'dMass (eau + neige atm) = {:12.3e} Sv '.format ( (dDYN_mas_wat + dATM_mas_sno)/dtime_sec*1E-6/ATM_rho) )
echo ( 'dMass (eau + neige atm) = {:12.3e} m  '.format ( (dDYN_mas_wat + dATM_mas_sno)/ATM_aire_sea_tot/ATM_rho) )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- SRF changes ' )

if Routing == 'SIMPLE' :
    RUN_mas_wat_beg = ONE_stock_int ( d_RUN_beg ['fast_reservoir'] +  d_RUN_beg ['slow_reservoir'] +  d_RUN_beg ['stream_reservoir'])
    RUN_mas_wat_end = ONE_stock_int ( d_RUN_end ['fast_reservoir'] +  d_RUN_end ['slow_reservoir'] +  d_RUN_end ['stream_reservoir'])

if Routing == 'SECHIBA' : 
    RUN_mas_wat_beg = ONE_stock_int ( d_SRF_beg['fastres']  + d_SRF_beg['slowres'] + d_SRF_beg['streamres'] \
                                    + d_SRF_beg['floodres'] + d_SRF_beg['lakeres'] + d_SRF_beg['pondres'] )
    RUN_mas_wat_end = ONE_stock_int ( d_SRF_end['fastres']  + d_SRF_end['slowres'] + d_SRF_end['streamres'] \
                                    + d_SRF_end['floodres'] + d_SRF_end['lakeres'] + d_SRF_end['pondres'] )

dRUN_mas_wat = RUN_mas_wat_end - RUN_mas_wat_beg
    
echo ( '\nWater content in routing ' )
echo ( 'RUN_mas_wat_beg = {:12.6e} kg | RUN_mas_wat_end = {:12.6e} kg '.format (RUN_mas_wat_end, RUN_mas_wat_end) )
echo ( 'dMass (routing)  = {:12.3e} kg '.format(dRUN_mas_wat) )
echo ( 'dMass (routing)  = {:12.3e} Sv '.format(dRUN_mas_wat/dtime_sec*1E-9) )
echo ( 'dMass (routing)  = {:12.3e} m  '.format(dRUN_mas_wat/OCE_aire_tot*1E-3) )

print ('Reading SRF restart')
tot_watveg_beg  = d_SRF_beg['tot_watveg_beg']  ; tot_watveg_beg  = tot_watveg_beg .where (tot_watveg_beg < 1E10, 0.)
tot_watsoil_beg = d_SRF_beg['tot_watsoil_beg'] ; tot_watsoil_beg = tot_watsoil_beg.where (tot_watsoil_beg< 1E10, 0.)
snow_beg        = d_SRF_beg['snow_beg']        ; snow_beg        = snow_beg       .where (snow_beg       < 1E10, 0.)

tot_watveg_end  = d_SRF_end['tot_watveg_beg']  ; tot_watveg_end  = tot_watveg_end .where (tot_watveg_end < 1E10, 0.)
tot_watsoil_end = d_SRF_end['tot_watsoil_beg'] ; tot_watsoil_end = tot_watsoil_end.where (tot_watsoil_end< 1E10, 0.)
snow_end        = d_SRF_end['snow_beg']        ; snow_end        = snow_end       .where (snow_end       < 1E10, 0.)

if LMDZ :
    tot_watveg_beg  = lmdz.geo2point (tot_watveg_beg)
    tot_watsoil_beg = lmdz.geo2point (tot_watsoil_beg)
    snow_beg        = lmdz.geo2point (snow_beg)
    tot_watveg_end  = lmdz.geo2point (tot_watveg_end)
    tot_watsoil_end = lmdz.geo2point (tot_watsoil_end)
    snow_end        = lmdz.geo2point (snow_end)

SRF_wat_beg = tot_watveg_beg + tot_watsoil_beg + snow_beg
SRF_wat_end = tot_watveg_end + tot_watsoil_end + snow_end

#SRF_mas_wat_beg = d_SRF_beg['tot_watveg_beg']+d_SRF_beg['tot_watsoil_beg'] + d_SRF_beg['snow_beg']
#SRF_mas_wat_end = d_SRF_end['tot_watveg_beg']+d_SRF_end['tot_watsoil_beg'] + d_SRF_end['snow_beg']

#if LMDZ :
#    tot_watveg_beg  = tot_watveg_beg .rename ( {'y':'points_phyiques'} )
#    tot_watsoil_beg = tot_watsoil_beg.rename ( {'y':'points_phyiques'} )
#    snow_beg        = snow_beg       .rename ( {'y':'points_phyiques'} )
#    tot_watveg_end  = tot_watveg_end .rename ( {'y':'points_phyiques'} )
#    tot_watsoil_end = tot_watsoil_end.rename ( {'y':'points_phyiques'} )
#    snow_end        = snow_end       .rename ( {'y':'points_phyiques'} )
#    SRF_wat_beg     = SRF_wat_beg    .rename ( {'y':'points_phyiques'} )
#    SRF_wat_end     = SRF_wat_end    .rename ( {'y':'points_phyiques'} )
if ICO :
    tot_watveg_beg  = tot_watveg_beg .rename ( {'y':'cell_mesh'} )
    tot_watsoil_beg = tot_watsoil_beg.rename ( {'y':'cell_mesh'} )
    snow_beg        = snow_beg       .rename ( {'y':'cell_mesh'} )
    tot_watveg_end  = tot_watveg_end .rename ( {'y':'cell_mesh'} )
    tot_watsoil_end = tot_watsoil_end.rename ( {'y':'cell_mesh'} )
    snow_end        = snow_end       .rename ( {'y':'cell_mesh'} )
    SRF_wat_beg     = SRF_wat_beg    .rename ( {'y':'cell_mesh'} )
    SRF_wat_end     = SRF_wat_end    .rename ( {'y':'cell_mesh'} )  

print ('Computing integrals')

print ( ' 1/6', end='' ) ; sys.stdout.flush ()
SRF_mas_watveg_beg   = ATM_stock_int ( tot_watveg_beg  )
print ( ' 2/6', end='' ) ; sys.stdout.flush ()
SRF_mas_watsoil_beg  = ATM_stock_int ( tot_watsoil_beg )
print ( ' 3/6', end='' ) ; sys.stdout.flush ()
SRF_mas_snow_beg     = ATM_stock_int ( snow_beg        )
print ( ' 4/6', end='' ) ; sys.stdout.flush ()
SRF_mas_watveg_end   = ATM_stock_int ( tot_watveg_end  )
print ( ' 5/6', end='' ) ; sys.stdout.flush ()
SRF_mas_watsoil_end  = ATM_stock_int ( tot_watsoil_end )
print ( ' 6/6', end='' ) ; sys.stdout.flush ()
SRF_mas_snow_end     = ATM_stock_int ( snow_end        )
print (' -- ') ; sys.stdout.flush ()

dSRF_mas_watveg   = SRF_mas_watveg_end   - SRF_mas_watveg_beg
dSRF_mas_watsoil  = SRF_mas_watsoil_end  - SRF_mas_watsoil_beg
dSRF_mas_snow     = SRF_mas_snow_end     - SRF_mas_snow_beg

echo ('\nLes differents reservoirs')
echo ( f'SRF_mas_watveg_beg   = {SRF_mas_watveg_beg :12.6e} kg | SRF_mas_watveg_end   = {SRF_mas_watveg_end :12.6e} kg ' )
echo ( f'SRF_mas_watsoil_beg  = {SRF_mas_watsoil_beg:12.6e} kg | SRF_mas_watsoil_end  = {SRF_mas_watsoil_end:12.6e} kg ' )
echo ( f'SRF_mas_snow_beg     = {SRF_mas_snow_beg   :12.6e} kg | SRF_mas_snow_end     = {SRF_mas_snow_end   :12.6e} kg ' )

echo ( 'dMass (watveg)  = {:12.3e} kg | {:12.2e} Sv | {:12.2e} m '.format (dSRF_mas_watveg , dSRF_mas_watveg /dtime_sec*1E-9, dSRF_mas_watveg /OCE_aire_tot*1E-3) )
echo ( 'dMass (watsoil) = {:12.3e} kg | {:12.2e} Sv | {:12.2e} m '.format (dSRF_mas_watsoil, dSRF_mas_watsoil/dtime_sec*1E-9, dSRF_mas_watsoil/OCE_aire_tot*1E-3) )
echo ( 'dMass (sno)     = {:12.3e} kg | {:12.2e} Sv | {:12.2e} m '.format (dSRF_mas_snow   , dSRF_mas_snow   /dtime_sec*1E-9, dSRF_mas_snow   /OCE_aire_tot*1E-3  ) )

SRF_mas_wat_beg = SRF_mas_watveg_beg + SRF_mas_watsoil_beg + SRF_mas_snow_beg 
SRF_mas_wat_end = SRF_mas_watveg_end + SRF_mas_watsoil_end + SRF_mas_snow_end
dSRF_mas_wat = SRF_mas_wat_end - SRF_mas_wat_beg

echo ( '\nWater content in surface ' )
echo ( 'SRF_mas_wat_beg  = {:12.6e} kg | SRF_mas_wat_end  = {:12.6e} kg '.format (SRF_mas_wat_beg, SRF_mas_wat_end) )
echo ( 'dMass (water srf) = {:12.3e} kg '.format (dSRF_mas_wat) )
echo ( 'dMass (water srf) = {:12.3e} Sv '.format (dSRF_mas_wat/dtime_sec*1E-6/ATM_rho) )
echo ( 'dMass (water srf) = {:12.3e} m  '.format (dSRF_mas_wat/ATM_aire_sea_tot/ATM_rho) )

echo ( '\nWater content in  ATM + SRF + RUN ' )
echo ( 'mas_wat_beg = {:12.6e} kg | mas_wat_end = {:12.6e} kg '.
           format (DYN_mas_wat_beg + ATM_mas_sno_beg + RUN_mas_wat_beg + SRF_mas_wat_beg,
                   DYN_mas_wat_end + ATM_mas_sno_end + RUN_mas_wat_end + SRF_mas_wat_end) )
echo ( 'dMass (water atm+srf+run) = {:12.6e} kg '.format ( dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat) )
echo ( 'dMass (water atm+srf+run) = {:12.3e} Sv '.format ((dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat)/dtime_sec*1E-6/ATM_rho) )
echo ( 'dMass (water atm+srf+run) = {:12.3e} m  '.format ((dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat)/ATM_aire_sea_tot/ATM_rho) )

echo ( '\n------------------------------------------------------------------------------------' )
echo ( '-- Change in all components' )
echo ( 'mas_wat_beg = {:12.6e} kg | mas_wat_end = {:12.6e} kg'.
           format (SEA_mas_wat_beg + DYN_mas_wat_beg + ATM_mas_sno_beg + RUN_mas_wat_beg + SRF_mas_wat_beg,
                   SEA_mas_wat_end + DYN_mas_wat_end + ATM_mas_sno_end + RUN_mas_wat_end + SRF_mas_wat_end) )
echo ( 'dMass (water CPL)         = {:12.3e} kg '.format ( dSEA_mas_wat + dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat) )
echo ( 'dMass (water CPL)         = {:12.3e} Sv '.format ((dSEA_mas_wat + dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat)/dtime_sec*1E-9) )
echo ( 'dMass (water CPL)         = {:12.3e} m  '.format ((dSEA_mas_wat + dDYN_mas_wat + dATM_mas_sno + dRUN_mas_wat + dSRF_mas_wat)/OCE_aire_tot*1E-3) )


